; Author: Leonie Drsechler-Fischer
(module lilies-module mzscheme
  (provide drawRecursively
           lilyLeaf olilyLeaf lily1 lily2 lily3 olily2 olily3
           square2 square3
           lilyLeafPause drawRecursivelyPause lily1Pause
          ) 


 (require (lib "trace.ss"))
 (require (lib "plt-pretty-big.ss" "lang"))
 (require (lib "draw.ss" "teachpack" "htdp"))
 (require (lib "tools-module.ss" "teachpack" "se3-bib"))
 (require (lib "macos-module.ss" "teachpack" "se3-bib"))
 (require (lib "my-vector-graphics.ss" "teachpack" "se3-bib"))
  
;;; lily
(define *defsize* 200)
  
(define (drawRecursively 
         shape ; the corners of the polyline, 
         ; a list of positions relative to the origin
         center; where to place the figure 
         howOften ; number of iterations
         rotationAngle ; initial rotation (degrees)
         rotationInc ; 
         scaleFactor ; initial scaleFactor between 0 and 1.0
         scaleInc;
         color
         )
  (if (> howOften 0)
      (let* ((rPoints (map ; rotate and scale the figure
                       (lambda (p)
                         (scale 
                          (rotate p rotationAngle) scaleFactor))
                       shape))
             (tPoints (map (rcurry translate center) rPoints))) 
        ;translate to the center
        (draw-polyline tPoints color)
        (drawRecursively 
         rPoints ; the corners of the polyline, 
         ; a list of positions relative to the origin
         center; where to place the figure 
         (- howOften 1); number of iterations
         (+ rotationAngle rotationInc) ; initial rotation (degrees)
         rotationInc ; 
         (* scaleFactor scaleInc) ; initial scaleFactor between 0 and 1.0
         scaleInc;
         color
         ))))
  
  (define (drawRecursivelyPause 
           ; wait for mouse click after drawing each polygon 
         shape ; the corners of the polyline, 
         ; a list of positions relative to the origin
         center; where to place the figure 
         howOften ; number of iterations
         rotationAngle ; initial rotation (degrees)
         rotationInc ; 
         scaleFactor ; initial scaleFactor between 0 and 1.0
         scaleInc;
         color
         )
  (if (> howOften 0)
      (let* ((rPoints (map ; rotate and scale the figure
                       (lambda (p)
                         (scale 
                          (rotate p rotationAngle) scaleFactor))
                       shape))
             (tPoints (map (rcurry translate center) rPoints))) 
        ;translate to the center
        (draw-polyline tPoints color)
        (wait-for-mouse-click)
        
        (drawRecursivelyPause 
         rPoints ; the corners of the polyline, 
         ; a list of positions relative to the origin
         center; where to place the figure 
         (- howOften 1); number of iterations
         (+ rotationAngle rotationInc) ; initial rotation (degrees)
         rotationInc ; 
         (* scaleFactor scaleInc) ; initial scaleFactor between 0 and 1.0
         scaleInc;
         color
         ))))

(define (lilyLeaf size iter dir sextant)
  ; draw closed triangles
  (let* ((c1 (rotate (scale *x* size) (* sextant 60)))
         (c2 (rotate c1 60))
         (m (scale (translate c1 c2) -1/3)) ;der Mittelpunkt von 0,c1,c2
         (c1r (translate c1 m))
         (c2r (translate c2 m))
         (o-r (translate *o* m))
         (canvasCenter (make-posn size size))
         (plotAt (translate canvasCenter (scale m -1))))
    ; (start/cartesian-plane (* 2 size) (* 2 size))
    (drawRecursively 
     (list c1r o-r c2r c1r) ; the corners of the polyline, 
     ; a list of positions relative to the origin
     plotAt; where to place the figure 
     iter; number of iterations
     0 ; initial rotation (degrees)
     (* dir (/ 120 iter)) ; 
     1.0 ; initial scaleFactor between 0 and 1.0
     0.995;
     'black
     )))

  (define (lilyLeafPause size iter dir sextant)
  ; draw closed triangles, wait for mouse click  after each triangle
  (let* ((c1 (rotate (scale *x* size) (* sextant 60)))
         (c2 (rotate c1 60))
         (m (scale (translate c1 c2) -1/3)) ;der Mittelpunkt von 0,c1,c2
         (c1r (translate c1 m))
         (c2r (translate c2 m))
         (o-r (translate *o* m))
         (canvasCenter (make-posn size size))
         (plotAt (translate canvasCenter (scale m -1))))
    ; (start/cartesian-plane (* 2 size) (* 2 size))
    (drawRecursivelyPause 
     (list c1r o-r c2r c1r) ; the corners of the polyline, 
     ; a list of positions relative to the origin
     plotAt; where to place the figure 
     iter; number of iterations
     0 ; initial rotation (degrees)
     (* dir (/ 120 iter)) ; 
     1.0 ; initial scaleFactor between 0 and 1.0
     0.995;
     'black
     )))
(define (olilyLeaf size iter dir sextant)
  ; draw open triangles
  (let* ((c1 (rotate (scale *x* size) (* sextant 60)))
         (c2 (rotate c1 60))
         (m (scale (translate c1 c2) -1/3)) ;der Mittelpunkt von 0,c1,c2
         (c1r (translate c1 m))
         (c2r (translate c2 m))
         (o-r (translate *o* m))
         (canvasCenter (make-posn size size))
         (plotAt (translate canvasCenter (scale m -1))))
    ; (start/cartesian-plane (* 2 size) (* 2 size))
    (drawRecursively 
     (list c1r o-r c2r) ; the corners of the polyline, 
     ; a list of positions relative to the origin
     plotAt; where to place the figure 
     iter; number of iterations
     0 ; initial rotation (degrees)
     (* dir (/ 120 iter)) ; 
     1.0 ; initial scaleFactor between 0 and 1.0
     0.995;
     'black
     )))

(define (lily1 iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size))
    (lilyLeaf size iter -1 4)))

  (define (lily1Pause iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size))
    (lilyLeafPause size iter -1 4)))
  
(define (lily2 iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size)) 
    (map (curry lilyLeaf size iter 1) '(0 1 2 3 4 5)))
  #t)

(define (lily3 iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size)) 
    (map (curry lilyLeaf size iter)
         '(1 -1 1 -1 1 -1)
         '(0 1 2 3 4 5)))
  #t)

(define (olily2 iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size)) 
    (map (curry olilyLeaf size iter 1) '(0 1 2 3 4 5)))
  #t)

(define (olily3 iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size)) 
    (map (curry olilyLeaf size iter)
         '(1 -1 1 -1 1 -1)
         '(0 1 2 3 4 5)))
  #t)

  (define (square-leaf size iter dir quadrant)
  ; draw closed triangles
  (let* ((c1 (rotate (scale *x* size) (* quadrant 90)))
         (c2 (rotate c1 90))
         (c3 (translate c1 c2))
         (m (scale (translate (translate c1 c2) c3) -1/4))
         ;der Mittelpunkt von c1,c2,c3, 0
         (c1r (translate c1 m))
         (c2r (translate c2 m))
         (c3r (translate c3 m))
         (o-r (translate *o* m))
         (canvasCenter (make-posn size size))
         (plotAt (translate canvasCenter (scale m -1))))
    ; (start/cartesian-plane (* 2 size) (* 2 size))
    (drawRecursively 
     (list c1r o-r c2r c3r c1r) ; the corners of the polyline, 
     ; a list of positions relative to the origin
     plotAt; where to place the figure 
     iter; number of iterations
     0 ; initial rotation (degrees)
     (* dir (/ 120 iter)) ; 
     1.0 ; initial scaleFactor between 0 and 1.0
     0.995;
     'black
     )))

  (define (square2 iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size)) 
    (map (curry square-leaf size iter 1) '(0 1 2 3)))
  #t)

(define (square3 iter)
  (let ((size *defsize*))
    (start (* 2 size) (* 2 size)) 
    (map (curry square-leaf size iter)
         '(1 -1 1 -1 )
         '(0 1 2 3 )))
  #t)

  ) ; end module lilies