(module maze-module swindle
  (provide search-maze  *basic-maze* *mini-maze*
           maze-state equal-state?
           maze-demo1 maze-demo2 maze-demo3 maze-demo4
           stop) 
  
  (require (lib "trace.ss"))
  (require (lib "draw.ss" "htdp"))
  
  (require (all-except 
            (lib "tools-module.ss" "teachpack" "se3-bib")
            mappend  every some last concat))  
  
  (require (lib  "my-vector-graphics.ss"  "teachpack" "se3-bib"))
  (require (lib  "backtracking-oo-module.ss" "teachpack" "se3-bib"))
  (require (lib  "macos-module.ss" "teachpack" "se3-bib"))
  (require (lib  "grid-map-module.ss"  "teachpack" "se3-bib"))
  
  ;;; =====================================================
  ;;; Suche den Weg aus einem Labyrinth
  ;;; =====================================================
  (define *entry-marker* #\E)
  (define *exit-marker* #\O)
  (define *center-marker* #\C)
  (define *path-marker* #\.)
  (define *hedge-marker* #\H)
  
  (define *maze-colours*
    (make-colour-map 
     (list #\E     #\.    #\O     #\C    #\H )
     (list 'yellow 'white 'white  'blue  'black)))
  
  (define *basic-maze*
    (vector 
     "HHHHHHHHHHHHHHHHHHHHHHEHHHHHHHHHHHHHHHHHHHHHH"
     "H...........................................H"
     "H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H"
     "H.H...................H...................H.H"
     "H.H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H.H"
     "H.H.H...................................H.H.H"
     "H.H.H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H.H.H"
     "H.H.H.H...............H...............H.H.H.H"
     "H.H.H.H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H.H.H.H"
     "H.H.H.H.H...........................H.H.H.H.H"
     "H.H.H.H.H.HHHHHHHHHHHHHHHHHHHHHHHHH.H.H.H.H.H"
     "H.H.H.H.H.H...........H...........H.H.H.H.H.H"
     "H.H.H.H.H.H.HHHHHHHHHHHHHHHHHHHHH.H.H.H.H.H.H"
     "H.H.H.H.H.H.H...................H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.HHHHHHHHCHHHHHHHH.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.......H.......H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.HHHHHHHHHHHHH.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H...........H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.HHHHHHHHH.H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.H...H...H.H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.H.HHHHH.H.H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.H.H...H.H.H.H.H.H.H.H.H.H.H"
     "O.H...H...H...H...H...C.H...H...H...H...H...O"
     "H.H.H.H.H.H.H.H.H.H.H...H.H.H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.H.HHHHH.H.H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.H.......H.H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.HHHHHHHHH.H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.H.....H.....H.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H.HHHHHHHHHHHHH.H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.H...............H.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.HHHHHHHHHHHHHHHHH.H.H.H.H.H.H.H"
     "H.H.H.H.H.H.H.........H.........H.H.H.H.H.H.H"
     "H.H.H.H.H.H.HHHHHHHHHHHHHHHHHHHHH.H.H.H.H.H.H"
     "H.H.H.H.H.H.......................H.H.H.H.H.H"
     "H.H.H.H.H.HHHHHHHHHHHHHHHHHHHHHHHHH.H.H.H.H.H"
     "H.H.H.H.H.............H.............H.H.H.H.H"
     "H.H.H.H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H.H.H.H"
     "H.H.H.H...............................H.H.H.H"
     "H.H.H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H.H.H"
     "H.H.H.................H.................H.H.H"
     "H.H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H.H"
     "H.H...................C...................H.H"
     "H.HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH.H"
     "H.....................H.....................H"
     "HHHHHHHHHHHHHHHHHHHHHOHOHHHHHHHHHHHHHHHHHHHHH"
     ))
  
  (define *mini-maze* (vector    
                       "HHHHHHEHHHHH"
                       "H..........H"
                       "HH.HHHHHHH.H"
                       "H...H......H"
                       "H.HHHHCHHHHH"
                       "H..........H"
                       "HHH.HHHH.HHH"
                       "H..C.H.....H"
                       "H.HHHHHHH.HH"
                       "H...H......H"
                       "HHOHHHHOHHHH"))
  
  
  
  (define (find-entry maze)
    ; Maze: a gridmap of hedges
    (let* ((rows (map-rows maze))
           (cols (map-cols maze)))
      (letrec ( (try-column (lambda (r c)
                              (let ((coord (make-map-coord r c)))
                                (cond ((> c cols) #f)
                                      ((is-entry? coord) coord)
                                      (else (try-column r (add1 c)))))))
                (try-row (lambda (r c)
                           (let ((coord (make-map-coord r c)))
                             (cond ((> r rows) #f)
                                   ((is-entry? coord) coord)
                                   (else (try-row (add1 r) c)))))))
        (or (try-row 1 1) 
            (try-column 1 1)
            (try-row rows 1) 
            (try-column cols 1))))) 
  
  
  
  (define (is-entry? coord)
    ; is the square at coord the entry to the maze?
    (equal? (map-field coord) *entry-marker*))
  
  (define (is-exit? coord)
    ; is the square at coord an exit to the maze?
    (let ((exit-found 
           (equal? (map-field coord) *exit-marker*)))
      (if exit-found (Fred "Found an exit"))
      exit-found ))
  
  (define (is-center? coord)
    ; is the square at coord an the center with the hidden treasure of the maze?
    (let ((treasure-found 
           (equal? (map-field coord) *center-marker*)))
      (if treasure-found (Fred "Found a treasure"))
      treasure-found ))
  
  
  (define (is-hedge? coord)
    (equal? (map-field coord) *hedge-marker*))

  (define (is-path? coord)
    ; clear path to walk
    (and (in-bounds? coord)
     (not (is-hedge? coord))))
  
  ; ==================================================
  ; drawing functions and constants
  ; ==================================================
  
  (define *seeker-color* 'red)
  (define *visited-color* 'yellow)
  (define *sleep-seconds* 0.4)
  
  ; ==================================================
  ; the states: a list of the positions visited so far
  ; ==================================================
  
  (defclass maze-state (state)
    ; a state of the backtracking search
    (maze-positions     
     :accessor current-position
     ; the current position in the maze
     :initarg :mPos
     ; :inivalue '()
     )
    
    :autopred #t ; auto generate predicate missionary?
    ; :automaker #t ; auto generates constructor make-m-state
    :printer  #t 
    )
  
  (defmethod initialize ((mazest maze-state) initargs)
    (call-next-method); initialize the slots from the super class
    ; retrieve the initargs by keywords
    
    (set-current-position! 
     mazest 
     (my-keyword-get initargs :mPos (always (void))))
    )
  
  (defmethod make-successor-state 
    ((prevState maze-state) this-position)
    (make (class-of prevState) 
          ; ensure the same class as the previous state
          :state-parent prevState
          :mPos this-position))
  
  (define (make-start-state theClass maze)
    ;find an entry to the maze and create the initial state
    ; theClass: the class of the search space
    ; maze: the layout of the maze
    (init-drawing 
       maze ; the pixmap image to be drawn
       *maze-colours* ; the colour table to use
       'black ; the drawing colour for additional lines 
       'white)
      (paint-map)

    (let ((some-entry (find-entry maze)))
      (if (not some-entry) 
          (error "the maze has no entry")
          (make theClass 
           :state-parent emptystate
           :mPos some-entry))))

  (defmethod show-state  ((mazest emptystate))
    ; draw the seeker at his position
    #t ; do nothing
    )
  
  (defmethod show-state  ((mazest maze-state))
    ; draw the seeker at his position
    (display-position (current-position mazest))
    (draw-plain-marker 
     (current-position mazest)
     *seeker-color*)
    ; go slowly, if the maze is small
    (if (< (map-cols *the-current-pixmap*) 15)
        (sleep-for-a-while *sleep-seconds*)))
  
  (defmethod clear-state ((mst emptystate))
    ; restore the background after the seeker has been moved
    #t ; do nothing if the start state
    )
  
  (defmethod clear-state ((mst maze-state))
    ; restore the background after the seeker has been moved
    (draw-plain-marker (current-position mst)
                       *visited-color*))
  
  
  (defmethod is-legal? ((mst maze-state))
    (is-path? (current-position mst)))

  (defmethod gen-states ((mazest maze-state))
    ; try one step in each of the four directions
    ; check if there is a hedge
    ; or if the position is outside the bounderies of the maze
    (let* ((this-position (current-position mazest))
           (this-row (map-coord-row this-position))
           (this-col (map-coord-col this-position))
           (north (make-map-coord (sub1 this-row) this-col))
           (south (make-map-coord (add1 this-row) this-col))
           (east (make-map-coord this-row (add1 this-col)))
           (west (make-map-coord this-row (sub1 this-col)))
           (next-positions ; try one step in each of the four directions
            (filter is-path?
                    (list east south west north)))
           (next-states 
            (map (curry make-successor-state mazest) 
                 next-positions)))
      next-states))
  
  
  (defmethod is-final? ((mst maze-state))
    ; searching the maze for an exit:
    ; is the position at an exit?
    (is-exit? (current-position mst)))
  
  (defmethod equal-state? ((mst1 maze-state)(mst2 maze-state))
               (equal-coord?
                (current-position  mst1)   
                (current-position  mst2)))

  (defmethod equal-state? ((mst1 emptystate)(mst2 emptystate))
  #t)
               
  (defclass maze-treasure-state (maze-state)
    ; searching a maze for treasures
    :autopred #t ; auto generate predicate missionary?
    ; :automaker #t ; auto generates constructor make-m-state
    :printer  #t 
    )
  (defmethod is-final? ((mst maze-treasure-state))
    ; searching a maze for hidden treasures:
    ; is there a treasure at the current position ?
    (is-center? (current-position mst)))
  
  (define (search-maze mstartstate)
    (let* ((success 
            (general-backtracking-non-cyclic-oo 
             mstartstate)); initial state
           (no-solutions (length success))) 
      (Fred (string-append 
             ; speak the number of solutions using Fred's voice
             (number->string no-solutions)
             (if (= 1 no-solutions) " solution" " solutions")
             " found"))
      (length success)
      ))
  
  (define (maze-demo1) 
    ; searching an exit of the mini-maze
    (search-maze 
     (make-start-state maze-state *mini-maze*)))
  
  (define (maze-demo2) 
    (search-maze 
     (make-start-state maze-state *basic-maze*)))
  
  (define (maze-demo3) 
    (search-maze 
     (make-start-state maze-treasure-state *mini-maze*)))
  
  (define (maze-demo4) 
    (search-maze 
     (make-start-state maze-treasure-state *basic-maze*)))
  
  ) ; end maze-moduleS