
(module my-vector-graphics  mzscheme
  (provide *x* *y* *-x* *-y* *o* 
           scale translate rotate rotateFixpoint 
           distance interpolate
           normal draw-polyline draw-poly) 


 (require (lib "trace.ss"))
 (require (lib "plt-pretty-big.ss" "lang"))
 (require (lib "draw.ss""teachpack" "htdp"))
 (require (lib "tools-module.ss" "teachpack" "se3-bib"))

;; arithmetic operations on positions

(define *x* (make-posn 1 0))
(define *y* (make-posn 0 1))
(define *-x* (make-posn -1 0))
(define *-y* (make-posn 0 -1))
(define *o* (make-posn 0 0))

(define (scale posn factor)
  (make-posn
   (* factor (posn-x posn))
   (* factor (posn-y posn))))

(define (translate posn trposn)
  (make-posn
   (+ (posn-x posn) (posn-x trposn))
   (+ (posn-y posn) (posn-y trposn))))

  ;(define (sqr x) (* x x)) ; in pretty Big schon drin
  
  
(define (rotate posn angle) ; angle degrees
  ;x'=x*cos(angle)-y*sin(angle)
  ;y'=y*cos(angle)+x*sin(angle)
  (let* ((angRad (* angle (/ pi 180.0)))
         (sinAng (sin angRad))
         (cosAng (cos angRad)))
    (make-posn
     (- (* (posn-x posn) cosAng) 
        (* (posn-y posn) sinAng))
     (+ (* (posn-y posn) cosAng)
        (* (posn-x posn) sinAng)))))

(define (rotateFixpoint posn angle center) ; angle degrees
  ;x'=x*cos(angle)-y*sin(angle)
  ;y'=y*cos(angle)+x*sin(angle)
  (let* ((angRad (* angle (/ pi 180.0)))
         (sinAng (sin angRad))
         (cosAng (cos angRad))
         (xf (-(posn-x posn)(posn-x center)))
         (yf (-(posn-y posn)(posn-y center))))
    (translate 
     (make-posn
      (- (* xf cosAng) (* yf sinAng))
      (+ (* yf cosAng) (* xf sinAng)))
     center)))

(define (distance from to) 
  ; euclidian distance between position from and to 
  (sqrt (+ (sqr (- (posn-x from) (posn-x to)))
           (sqr (- (posn-y from) (posn-y to))))))

(define (interpolate pos-1 pos-2 fract)
  ; compute a position between pos-1 and pos-2
  (let* ((dx  (- (posn-x pos-2) (posn-x pos-1)))
         (dy  (- (posn-y pos-2) (posn-y pos-1))))
    (make-posn
     (+ (posn-x pos-1)
        (* dx fract))
     (+ (posn-y pos-1)
        (* dy fract)))))

(define (normal pos-1 pos-2 nlength)
  ; compute a normal vector to the line from pos-1 to pos-2
  (let* ((dx  (- (posn-x pos-2) (posn-x pos-1)))
         (dy  (- (posn-y pos-2) (posn-y pos-1)))
         (dist (sqrt (+ (sqr dx) (sqr dy))))
         )
    (make-posn
     (-(* dy nlength (/ 1 dist) ))
     (* dx nlength  (/ 1 dist)))))


(define (draw-polyline positions color)
  ; draw a polygon given as a list of points
  (if (> (length positions) 1)
      (begin 
        (draw-solid-line (car positions) (cadr positions) color)
        (draw-polyline (cdr positions) color))))

(define (draw-poly positions color)
  ; draw a closed polygon given as a list of points, 
  ; connect the last point to the first point
  (if (> (length positions) 1)
      (begin 
        (draw-solid-line ; close the polygon
         (car positions) 
         (last positions) 
         color)
        (draw-polyline positions color))))

  ) ; end module my-vector-graphics