(module newton-module mzscheme
  (provide deriv newton
           m-sqrt sin-abl sin-2abl cubrt root n-sqrt
          ) 
  (require (lib "trace.ss"))
  (require (lib "tools-module.ss" "teachpack" "se3-bib"))

(define (deriv f)
  ;;; Construct the derivative function of f
  (let ((dx 0.0001))
    (lambda (x)
      (/ (- (f (+ x dx)) (f x))
         dx))))

(define sin-abl (deriv sin))
(define sin-2abl (deriv sin-abl))

(define (m-sqrt x)
  (let* ((eps 0.00001)
	 (good-enough? (lambda (y)
			(< (abs (- (square y) x)) 
			   eps)))
	 (improve (lambda (y)
		    (/ (+ y (/ x y)) 
		       2))))
    (untilM improve good-enough? x)))
		 
(define (newton f)
  ;;; return a function that iterates the newton method
  ;;; to find a root of f, starting with 'first guess'.
  (lambda (first-guess)
    (let* ((eps 0.00001)
	   (good-enough? (lambda (y)
			   (< (abs (f y)) eps)))
	   (improve (lambda (y)
		      (- y (/ (f y)
			      ((deriv f) y)))))) 
	   (untilM improve good-enough? first-guess))))
		 
(define (n-sqrt x)
  ;;; compute the sqrt of x using the general newton method
  ;;; solving: y*y-x = 0.
  ((newton 
     (lambda (y) 
        (- (square y) x)))
     x))

(define (cubrt x)
  ;;; the cubic root of x; solving y*y*y-x=0.
  ((newton 
     (lambda (y) (- (expt y 3) x)))
     x))

(define (root x n)
  ;;; the nth root of x
  ((newton 
     (lambda (y) (- (expt y n) x)))
     x))
  )