(module tools-module swindle
  (provide ;
   
   ; import from scheme/base
   compose foldl foldr filter filter-not
   
   ; scheme/list
  ; take drop
   
   pi ; import from math.ss 
   
  ;from  (require scheme/function)
   negate curry curryr

   add1 sub1 id square writeln last
   random-real random random-elt one-of
   untilM iterate reduce reduce-right filter 

   always rcurry  negate
   disjoin conjoin nats-1-n
   != odd even
   sublis subtree  drop subseq 
   position listOfNItems 
   concat ;flatten 
   mklist mappend atom?
   some every someLisp
   mysort minimumListe minElement vertauscheCarMinimum
   split merge merge-sort
   
   fib fac memo
   
   head-stream tail-stream cons-stream empty-stream? cons-stream2
   the-empty-stream stream-ref
   map-stream filter-stream for-each-stream
   take-stream drop-stream force-stream
   append-streams mappend-stream combine-all-streams
   print-stream integers-from-n 
   divisible? not-divisible?
   no-sevens fibs ints
   sieve
   
   my-delay my-force type-of space klingeling
   
   ;  macros
   ensure-car ; ensure arg1 is a proper list
   ;            before evaluating arg2
   <0>; numerical three way test: arg1 <0, =0, >0
   
   amblist ; non deterministic picking of alternatives
   my-keyword-get; retrieve arguments by keywords
   )  
  ;;;; =====================================================================
  ;;;; some basic functions that are missing in some scheme implementations,
  ;;;; uncomment, if necessary 
  ;;;; ======================================================================

  (require
   (all-except scheme/function 
               negate
               take); defined in swindle
   (all-except scheme/list flatten)
   (lib "setf.ss" "swindle")
  ; (lib "misc.ss" "swindle")
   (lib "trace.ss")
   (lib "math.ss")
   )

  
  (define (id x) x)
  
  (define (square x) (* x x))
  
  (define writeln 
    (lambda args
      (for-each display args)
      (newline)))
  
  
  ;;;; ==========================================================================
  ;;;; random numbers
  ;;;; =====================================================================
  
  (define *last-ran* 1) ; the previous random number
  (define (random-real)
    ;;; pick a random real number between 0 and 1.0
    (let* ((a (add1 (expt 2 7)))
           (b 1)
           (T (sub1 (expt 2 35)))
           (next-ran (remainder 
                      (+ (* a *last-ran*)
                         b)
                      T)))
      (set! *last-ran* next-ran)
      (/ next-ran T)))
  
  ;(define (random n)
  ;;; pick a random integer between 0 and n-1
  ; (inexact->exact (floor (* n (random-real)))))
  
  ;;; Choose an element from a list at random
  (define (random-elt choices)
    (list-ref choices 
              (random (length choices))))
  
  (define (one-of set)
    ;;;Pick one element of set, and make a list of it.
    (list (random-elt set)))
  
  ;;;; ==================================================================
  ;;;; Repetition
  ;;;; ==================================================================
  
  (define (iterate f end? seed)
    ; apply f to seed and repeat the process on the sequence of results (f (f (.. seed)))
    ; until (end? (f(f ..seed))) is satisfied. return the list of values.
    (letrec ((iteration 
              (lambda (result nth-term)
                (let ((new-term (f nth-term)))
                 
                  (if (end? nth-term) result
                      (iteration 
                       (cons new-term result)
                       new-term))))))
      (reverse 
       (iteration (list seed) seed))))
  
  (define (reduce f xs seed)
    (cond ((null? xs) seed)
          (else
           (reduce f 
                   (cdr xs)
                   (f (car xs) seed)))))
  
  (define (reduce-right f xs seed)
    ; allgemein rekurive Reduktion
    (if (null? xs)
        seed
        (f (car xs) 
           (reduce-right  f (cdr xs) seed))))
  
  
  
  ;;;; =========================================================================
  ;;;; The Dylan function builders, see Graham 96 for a Common Lisp Version
  ;;;; =========================================================================
  (define (always1 x)
    ;;; generate a function of one argument y returning constantly x.
    (lambda (y) x))
  
  (define (always x)
    ;;; generate a function with an arbitrary number of arguments returning constantly x.
    (lambda args x))
  
  (define (curry1 f x) 
    ;; curry left arg to the function f
    (lambda (y) (f x y)))
  
  (define rcurry curryr)
  
  (define (compose2 f g)
    ;; compose two functions in one argument: (f (g x))
    (lambda (x) (f (g x))))	
  
  (define disjoin 
    ;; return a composite predicate that is true 
    ;; when any of the component  predicates are true
    (lambda functions
      (if (null? functions) (always #f) ;no predicate is true
          (let ((f0 (car functions))
                (rest (cdr functions)))
            (lambda args
              (or (apply f0 args)
                  (apply (apply disjoin rest) args)))))))
  
  (define conjoin 
    ;; return a composite predicate that is true 
    ;; when all of the component  predicates are true
    (lambda functions
      (if (null? functions) (always #t) ;all predicates are true
          (let ((f0 (car functions))
                (rest (cdr functions)))
            (lambda args
              (and (apply f0 args)
                   (apply (apply disjoin rest) args)))))))
  
  (define (nats-1-n n)
    ; make a list of 1 .. n, n natural 
    (iterate (curry + 1)
             (curry = n)
             1))
  (define != (compose not =)) 
  
  (define (odd n)
    (cond ((= 0 n) #f)
          ((< n 0) (odd (abs n)))
          (else (even (- n 1)))))
  
  (define (even n)
    (cond ((= 0 n) #t)
          (else (odd (- n 1)))))
  
  ;;;; =============================================================================================
  ;;;;               list processing tools
  ;;;; =============================================================================================
  
  ;;;; substitute elements of a list according to an association list with mappings
  ;;;; (sublis '( (1 . a) (2 . b) (3 . c))
  ;;;;         '( 1 2 3 1 2 4 3))  => (a b c a b 4 c)
  
  (define (atom? v)
    (not (pair? v)))
  

  
  (define (subseq xs from to)
    ; the sublist of xs starting by element 'from'
    ; and ending by elelmt 'to', zero indexed
    ;bugfix: former versions of take and drop had a different order of the arguments
    (take (drop xs from)
          (add1 (- to from)) 
          ))
  
  (define (position x xs start)
    ; at which position does x occur in xs (skipping up to start elements)?
    (letrec 
        ((locate
          (lambda (x xs pos)
            (cond ((null? xs) #f)
                  ((equal? x (car xs)) pos)
                  (else (locate x (cdr xs) (+ 1 pos)))))))
      (locate x (drop xs start) start)))
  
  (define (sublis subs xs)
    ; substitute the elements of a linear list
    (let ((replacement 
           (lambda (x)
             (let ((found (assoc x subs)))
               (if (not found) x
                   (cdr found))))))
      (map replacement xs)))
  
  (define (subtree subs xs)
    ; substitute the leaves of a tree
    (let ((replacement 
           (lambda (x)
             (let ((found (assoc x subs)))
               (if (not found) x
                   (cdr found))))))
      (maptree replacement xs)))
  
  (define (listOfNItems n item)
    ; make a list of 1..n copiys of item
    (if (zero? n) '()
        (map (lambda (it) item)
             (nats-1-n n))
        ))
  
  (define (mklist x)
    "Return x if it is a list, otherwise (x)."
    (if (list? x)
        x
        (list x))) 
  
  
  ;;;; =============================================================================================
  ;;;; Memoization
  ;;;; =============================================================================================
  (define (fib n)
    "compute the n-th fibonacci number"
    (cond ((= 0 n) 1)
          ((= 1 n) 1)
          (else ( + (fib (- n 1))
                    (fib (- n 2))))))
  
  (define (fac n)
    (if (= n 0) 1 (* n (fac (- n 1)))))
  
  (define (memo fn)
    "Return a memo-function of fn."
    (letrec 
        ((table '())
         (store 
          (lambda (arg val) 
            (set! table (cons (cons arg val) table))
            val))
         (retrieve 
          (lambda (arg) 
            (let ((val-pair (assoc arg table)))
              (if val-pair (cdr val-pair) #f))))
         (ensure-val 
          (lambda (x)
            (let ((stored-val (retrieve x)))
	      (if stored-val stored-val
		  (store x (fn x)))))))
      ensure-val)) 
  ;;;; =============================================================================================
  ;;;; Stroeme
  ;;;; =============================================================================================
  ;;; implementing stream functions in scheme
  
  ;; empty-stream?
  (define (empty-stream? stream)
    (null? stream))
  
  ;; is-stream?
  (define (is-stream? stream)
    (cons? stream))
  
  ;; take first element from stream
  (define (head-stream stream)
    (car stream))
  
  ;; evaluate the rest of a stream
  (define (tail-stream stream)
    (cond ((null? stream) '())
          ((null? (cdr stream)) '())
          ((pair? (cdr stream)) (cdr stream))
          (else (force (cdr stream)))))
  
  ;; construct a stream: cons-stream must be a macro, since b must not be evaluated
  (define-syntax cons-stream 
    (syntax-rules ()
      ((cons-stream a b)
       (cons a (delay b)))))
  
  (defmacro (cons-stream2 a b)
    `(cons ,a (delay ,b)))
  
  (define the-empty-stream 
    '())
  
  (define (stream-ref stream i)
    "The i-th element of a stream, 0-based"
    (cond ((null? stream) #f)
          ((= 0 i) (head-stream stream))
          (else (stream-ref 
                 (tail-stream stream) (- i 1)))))
  
  ;;;=================================
  ;;; higher order stream functions
  
  ;; map each element of a stream
  (define (map-stream proc stream)
    (if (empty-stream? stream)
        the-empty-stream
        (cons
	 (proc (head-stream stream))
	 (delay 
           (map-stream 
            proc (tail-stream stream))))))
  
  ;; filter stream
  (define (filter-stream pred? stream)
    (cond ((empty-stream? stream) the-empty-stream)
          ((pred? (head-stream stream))
           (cons
            (head-stream stream)
            (delay 
              (filter-stream
               pred? (tail-stream stream)))))
          (else (filter-stream
                 pred? (tail-stream stream)))))
  
  ;; side effects
  ;; for-each
  (define (for-each-stream proc stream)
    (cond ((empty-stream? stream)
           'done)
          (else (proc (head-stream stream))
                (for-each-stream 
                 proc (tail-stream stream)))))
  
  (define (take-stream n stream)
    ; the sublist of the first n elements of xs
    (cond 
      ((empty-stream? stream) '())
      ((= 0 n) '())
      (else 
       (cons (head-stream stream)
             (take-stream 
              (- n 1) 
              (tail-stream stream))))))
  
  (define (force-stream-to-depth n depth stream)
    ; force the first n elements of the stream to depth "depth"
    (cond 
      ((not(is-stream? stream)) stream)
      ((empty-stream? stream) '())
      ((= 0 n) stream)
      (else 
       (cons 
        (force-stream-to-depth 
         depth depth (head-stream stream))
        (force-stream-to-depth  
         (- n 1)depth (tail-stream stream))))))
  
  (define (force-stream n stream)
    ; force the first n elements of the stream
    (force-stream-to-depth  n n stream))
  
  (define (drop-stream n stream)
    ; drop the first n elements of xs and return the rest
    (cond 
      ((empty-stream? stream) '())
      ((= 0 n) stream)
      (else (drop-stream 
             (- n 1)
             (tail-stream stream)))))
  
  (define  (append-streams x y)
    "Return a stream that appends the elements of x and y."
    (if (empty-stream? x)
        y
        (cons (head-stream x)
              (delay (append-streams (tail-stream x) y)))))
  
  (define (mappend-stream fn stream)
    "Lazily map fn over stream, appending results."
    (if (empty-stream? stream)
        the-empty-stream
        (let ((x (fn (head-stream stream))))
          (cons
           (head-stream x)
           (delay (append-streams 
	           (tail-stream x)
                   (mappend-stream
                    fn (tail-stream stream))))))))
  
  (define (combine-all-streams xstream ystream)
    "Return a stream of streams formed by appending an x to an y."
    ; in other words: form the cartesian product.
    (mappend-stream
     (lambda (y)
       (map-stream 
        (lambda (x)
          (append-streams x y))
        xstream))
     ystream))
  
  ;;;===================================
  ;;; examples
  (define (write-with-space x)
    (write x) (display " "))
  
  (define (print-stream stream)
    (for-each-stream write-with-space stream)
    (writeln))
  
  (define (integers-from-n n)
    (cons-stream n
                 (integers-from-n (+ 1 n)))) 
  
  (define (divisible? x y)
    (= (remainder x y) 0))
  
  (define ints (integers-from-n 1))
  
  (define no-sevens
    (filter-stream
     (lambda (x)
       (not (divisible? x 7)))
     ints))
  
  (define fibs-mapped
    ; fibonaci-numbers by mapping the integers
    (map-stream fib ints))
  
  (define (fibstream a b)
    ; fibonnaci-numbers using accumulators
    (let ((nfib (+ a b)))
      (cons-stream nfib (fibstream  b nfib))))
  
  (define fibs (cons-stream 1(fibstream 0 1)))
  
  ;;;;;;; implementing delay and force
  ;;
  (define (my-delay exp)
    (lambda () exp))
  
  ;;
  (define (my-force delayed-object)
    (delayed-object))
  
  (define (not-divisible? x y)
    (not (= 0 (remainder x y))))
  
  (define (sieve stream)
    (cons 
     (head-stream stream)
     (delay 
       (filter-stream 
        (rcurry
         not-divisible? 
         (head-stream stream))
        (sieve (tail-stream stream))))))
  
  (define (type-of x)
    (cond ((boolean?  x) 'boolean)
          ((pair? x) 'pair)
          ((list? x) 'list)
          ((symbol? x) 'symbol)
          ((number? x) 'number)
          ((char? x) 'char)
          ((string? x) 'string)
          ((vector? x) 'vector)
          ((procedure? x) 'procedure)
          (else 'promise)))
  
  (define (space n) 
    (untilM 
     (curry string-append " ")
     (compose (curry = n)
              string-length)
     ""))
  
  (define (klingeling wieoft)
    (untilM (lambda (x)
              (display (integer->char 7))
              (+ 1 x))
            (lambda (y)
              (= y wieoft))
            1))
  
  
  (define (minElement p<? a b)
    ;;; das Kleinere von a und b bezueglich p<?
    (if (p<? a b) a b))
  
  (define (minimumListe p<? xs)
    ;; not null? xs
    (let ((seed (car xs)))
      (reduce (curry minElement p<?) (cdr xs) (car xs))))
  
  (define (vertauscheCarMinimum p<? xs)
    ;;; vertausche das erste Element mit dem kleinsten Element der Liste
    ;;; not null? xs
    (if (null? (cdr xs)) 
        xs
        (let* ((mini (minimumListe p<? xs))
               (nachvorn (filter (curry equal? mini) xs))
               (nachhinten (filter (compose not (curry equal? mini)) xs)))
          (append nachvorn nachhinten))))
  
  
  (define (mysort p<? xs)
    ;;; sortiere eine Liste entsprechend einer Relation p<?
    ;;; Eine leere Liste ist sortiert.
    ;;; rekursiver Ansatz: eine Liste ist sortiert, wenn am Anfang das 
    ;;; kleinste Element steht und der Rest der Liste sortiert ist.
    
    ;;; Dazu wird das kleinste Element der Liste gesucht 
    ;;; und mit dem ersten Element vetauscht. 
    ;;;;Dann wird der Rest rekursiv sortiert.
    (cond ((null? xs) '())
          ((null? (cdr xs)) xs)
          (else 
           (let ((sxs (vertauscheCarMinimum p<? xs)))
             (cons (car sxs)(sort p<? (cdr sxs)))))))
  
  
  ;;;; effizienter: mergesort
  (define (split haelfte1 haelfte2 zuTeilen)
    ;zerlege eine Liste in zwei Haelften
    (if (null? zuTeilen) 
        (list haelfte1 haelfte2); resultat: die beiden Teillisten
        (split (cons (car zuTeilen) haelfte2); akkus abwechselnd fuellen
               haelfte1
               (cdr zuTeilen))))
  
  (define (merge p<? as bs)
    ;; p<? Ordnungsrelation
    ;; as, bs sortierte Teillisten
    (cond ((null? as) bs)
          ((null? bs) as)
          ((p<? (car as) (car bs))
           (cons (car as) 
                 (merge p<? (cdr as) bs)))
          (else (cons (car bs) 
                      (merge p<? (cdr bs) as)))))
  
  (define (merge-sort  p<? xs)
    (cond ((null? xs) '())
          ((null? (cdr xs)) xs)
          (else
           (let* ((theSplit (split '() '() xs))
                  (haelfte1 (car theSplit))
                  (haelfte2 (cadr theSplit)))
             ;(display xs)
             ; (display haelfte1)
             ; (display haelfte2)
             (merge p<?
                    (merge-sort  p<? haelfte1)
                    (merge-sort  p<? haelfte2))))))
  
  
  ; non determinism
  (define-syntax amblist
    ; return non deterministically one element from l
    (syntax-rules ()
      ((amb l) (eval `(amb ,@l)))
      ))
  
  ;    (if (null? l) (amb); für sehr lange listen
  ;        (amb (car l)
  ;             (amblist(cdr l)))))
  
  ; syntax utilities
  
  (define-syntax ensure-car 
    ; evaluate expr only if l is a proper list
    (syntax-rules ()
      ((ensure-car l expr) 
       (if (pair? l) expr #f))
      ))
  
  (define-syntax <0>
    ; three way case : <0, ==, >0
    (syntax-rules ()
      ((<0> num case<0 case=0 case>0)
       (cond ((< num 0) case<0)
             ((= num 0) case=0)
             ((> num 0) case>0)))))
  
  
  ; bug-fix: keyword-get in "kw.ss" does not seem to be reliable
  (define (my-keyword-get initargs theKey notfoundThunk)
    ; initargs are represented 
    ; as a list of alternating keys and values
    (let ((keylist (member theKey initargs)))
      (if (and keylist (> (length keylist) 1)) 
          (cadr keylist)
          (notfoundThunk)))
    ); end keyword-get
  
  ;;; ====================================================
  ;;; legacy definitions: previous version of the toolbox
  ;;; ====================================================
 
  (define (tbLast xs) ; provided by swindle
    (car (reverse xs)))
  
    (define (tbTake n xs)
    ; the sublist of the first n elements of xs: take from scheme/list
    (cond 
      ((null? xs) '())
      ((= 0 n) '())
      (else (cons (car xs)
                  (tbTake (- n 1) (cdr xs))))))

  (define (tbDrop n xs)
    ; drop the first n elements of xs and return the rest
    ; drop from scheme/list
    (cond 
      ((null? xs) '())
      ((= 0 n) xs)
      (else (tbDrop (- n 1) (cdr xs)))))
  
  (define tbRcurry
    ;;; curry an arbitrary number of args (as the last args)
    ;;; to a function of several args.
    (lambda args ;function name and args to be curried
      (let ((f (car args))
            (curried-args (cdr args)))
        (lambda not-curried-args
          (apply f 
                 (append
                  not-curried-args
                  curried-args))))))
  
   (define tbCurry
    ;;; curry an arbitrary number of args (from left to right)
    ;;; to a function of several args.
    (lambda args ;function name and args to be curried
      (let ((f (car args))
            (curried-args (cdr args)))
        (lambda not-curried-args
          (apply f 
                 (append
                  curried-args
                  not-curried-args))))))
  
    (define (untilM f end? seed)
    ; the until function of Miranda
    ; apply f to seed and repeat the process on the sequence of results (f (f (.. seed)))
    ; until (end? (f(f ..seed))) is satisfied. return the last value.
    
    (if (end? seed) seed
        (untilM f end? (f seed))))
  
  (define (tbFilter p? xs)
  (cond ((null? xs) '()) ; fertig
        ((p? (car xs))
           (cons (car xs) 
                 (tbFilter p? (cdr xs))))
        (else (tbFilter p? (cdr xs)))))
; filter: schon in pretty-big enthalten
  
  (define tbCompose 
  ;; compose an arbitrary number of functions 
  (lambda functions
    (if (null? functions) id
	(let* ((rfs (reverse functions))
	       (f0  (car rfs))) 
	  ; the last function has to be applied first
	  (lambda fargs
	    (reduce
	     (lambda (v fi) (fi v)) 
	     ; apply function i to the previous result
	     (cdr rfs)
	     (apply f0 fargs))))))) ; the seed value
; compose: schon in pretty-big enthalten
  
   (define (tbConcat xss) ; concat: provided by swindle
    ;; concatenate the sublists of a list of list
    (apply append xss))
 
  ; flatten: aus scheme/list
(define (tbFlatten ls)
    (cond  ((null? ls) '())
           ((pair? (car ls))
            (append (tbFlatten (car ls))
                    (tbFlatten (cdr ls))))
           ((null? (car ls)) (tbFlatten (cdr ls)))
           (else (cons (car ls)
                       (tbFlatten (cdr ls))))))

;;; (tbFlatten '(1 (2 (3 (4 ))))) => (1 2 3 4)
  
  (define (tbMappend fn the-list)	; mappend provided by swindle
    "Apply fn to each element of list and append the results."
    (apply append (map fn the-list)))
  
  (define (someLisp p? xs);some provided by swindle
    ;; does some element of xs satisfy p?
    ;; defined as semipredicate as in common Lisp
    (if (null? xs) #f
        (let ((p (p? (car xs))))
          (if p p
              (someLisp p? (cdr xs))))))
  
  (define (tbEvery p? xs); provided by swindle
    ;; does each element of xs satisfy p?
    (if (null? xs) #t
        (reduce 
         (lambda (x1 x2)
           (and x1 (p? x2)))
         (cdr xs)
         (p? (car xs)))))
  ); end module
